#version 450

layout (location = 0) in vec3 in_position;

layout (location = 1) in vec3 in_normal;

layout (binding = 0) uniform UniformBuffer {
	vec4 color;
	mat4 modelViewProjMatrix;
	ivec4 userInput;

} ub_data;

layout (binding = 1) uniform DirectionLight {
    vec4 direction;
    vec4 color;

} direction_data;

layout (binding = 2) uniform PointLight {
    vec4 position;
    vec4 color;
    vec4 attenuation;

} point_data;


layout (location = 0) out vec4 out_color;
layout (location = 1) out vec3 outNormal;

void main()
{
	gl_Position = ub_data.modelViewProjMatrix * vec4(in_position, 1.0);

	outNormal = in_normal;

    vec3 ambientLightIntensity = vec3(1, 1, 1); 
    float ambientStrength = 0.1; // Ambient light strength
    vec3 ambient = ambientStrength * ambientLightIntensity;

    vec3 norm = normalize(in_normal);
    vec3 lightDir = normalize(direction_data.direction.xyz);
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = diff * direction_data.color.rgb;

    float specularStrength = 0.5;
    vec3 viewDir = normalize(-in_position); 
    vec3 reflectDir = reflect(-lightDir, norm);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
    vec3 specular = specularStrength * spec * direction_data.color.rgb;


    vec3 pointLightDir = normalize(point_data.position.xyz - in_position);
    float pointDiff = max(dot(norm, pointLightDir), 0.0);
    vec3 pointDiffuse = pointDiff * point_data.color.rgb;

    // Attenuation
    float distance = length(point_data.position.xyz - in_position);
    float attenuation = 1.0 / (point_data.attenuation.x + point_data.attenuation.y * distance + point_data.attenuation.z * (distance * distance));
    pointDiffuse *= attenuation;

    // Add point light's diffuse to overall diffuse
    diffuse += pointDiffuse;

    vec3 phongColor = ambient + diffuse + specular;
    // Mix factor (you can adjust this value or make it uniform to control it via application)
    float mixFactor = 0.5;

    // Final color is a mix of the original color and the Phong illumination color
    vec3 finalColor = mix(ub_data.color.rgb, phongColor, mixFactor);

    out_color = vec4(finalColor, 1.0);




}
