#version 450
/*
 * Copyright 2023 TU Wien, Institute of Visual Computing & Human-Centered Technology.
 * This file is part of the GCG Lab Framework and must not be redistributed.
 *
 * Original version created by Lukas Gersthofer and Bernhard Steiner.
 * Vulkan edition created by Johannes Unterguggenberger (junt@cg.tuwien.ac.at).
 */

layout (location = 0) in VertexData {
    vec3 fragPositionWS;
    vec2 textureCoordinates;
} frag_in;

layout(binding = 0) uniform UniformBuffer {
    mat4 viewProjMatrixShadow;
    vec4 spotlightPositionWS; //w = farPlane
	vec4 spotlightDirectionWS;
    vec4 color;
    mat4 modelViewProjMatrix;
    mat4 modelMatrix;
    vec4 cameraPosition;
    ivec4 userInput;
    float ambientConstant;
    float diffuseConstant;
    float specularConstant;
    float shininessConstant;
} ub_data;

layout(binding = 1) uniform sampler2D diffuse_texture;

layout (location = 0) out vec4 out_color;

// for debug: makes shadow map better visible
float LinearizeDepth(float depth)
{
  float n = 0.001;
  float f = 20;
  float z = depth;
  return (2.0 * n) / (f + n - z * (f - n));	
}

void main() {
    if (0 == ub_data.userInput[3]) 
    {
        // Shade surfaces
        out_color = vec4(texture(diffuse_texture, frag_in.textureCoordinates).rgb , 1);
        
        // For debugging: This displays the shadowmap
        //float depthNormalized = texture(diffuse_texture, frag_in.textureCoordinates).r;
        //out_color = vec4(vec3(1.0-LinearizeDepth(depthNormalized)), 1.0);
    } 
    else 
    {
        // Set parameters for cross
        ivec2 resolution = ivec2(ub_data.userInput.y, ub_data.userInput.z);
        vec2 screenCenter = vec2(float(resolution.x) / 2, float(resolution.y) / 2);
        float verticalLineHalfThickness = 0.51;      // Must be >0.5
        float horizontalLineHalfThickness = 0.51;    // Must be > 0.5
        int crossHalfWidth = 30;                     // Must be > 0.5
        int crossHalfHeight = 30;                    // Must be > 0.5

        // Check if current fragment lies on cross
        bool fragmentOnHorizontalCrossLine = screenCenter.x - crossHalfWidth < gl_FragCoord.x && gl_FragCoord.x < screenCenter.x + crossHalfWidth
            && screenCenter.y - horizontalLineHalfThickness < gl_FragCoord.y && gl_FragCoord.y < screenCenter.y + horizontalLineHalfThickness;
        bool fragmentOnVerticalCrossLine = screenCenter.x - verticalLineHalfThickness < gl_FragCoord.x && gl_FragCoord.x < screenCenter.x + verticalLineHalfThickness
            && screenCenter.y - crossHalfHeight < gl_FragCoord.y && gl_FragCoord.y < screenCenter.y + crossHalfHeight;

        // If fragment lies on cross, draw black; otherwise, discard
        if (fragmentOnHorizontalCrossLine || fragmentOnVerticalCrossLine) {
            out_color = vec4(1, 0, 0, 1);
        } else {
            discard;
        }
        
    }
}
