#version 450

layout(location = 0) in vec3 inPosition;
layout(location = 1) in vec3 inNormal;
layout(location = 2) in vec2 inTexCoord;
layout(location = 3) in vec3 inTangent;

layout(location = 0) out VertexData {
    vec3 fragPositionWS;
    vec3 fragNormalWS;
    vec2 textureCoordinates;
    vec3 fragTangentWS;
    vec3 fragBitangentWS;
    vec4 fragPositionLightSpaceCS;
} frag_out;

layout(binding = 0) uniform UniformBuffer {
    mat4 viewProjMatrixShadow;
    vec4 spotlightPositionWS; //w = farPlane
	vec4 spotlightDirectionWS;
    vec4 color;
    mat4 modelViewProjMatrix;
    mat4 modelMatrix;
    vec4 cameraPosition;
    float ambientConstant;
    float diffuseConstant;
    float specularConstant;
    float shininessConstant;
} ub_data;

void main() {
    // Calculate world position
    vec4 worldPosition = ub_data.modelMatrix * vec4(inPosition, 1.0);
    frag_out.fragPositionWS = worldPosition.xyz;

    // Calculate world normal
    frag_out.fragNormalWS = mat3(ub_data.modelMatrix) * inNormal;

    vec3 worldTangent = normalize(mat3(ub_data.modelMatrix) * inTangent);
    frag_out.fragTangentWS = worldTangent;

    vec3 worldBitangent = normalize(cross(frag_out.fragNormalWS, worldTangent));
    frag_out.fragBitangentWS = worldBitangent;

    // Pass texture coordinates
    frag_out.textureCoordinates = inTexCoord;

    // Calculate clip space position
    gl_Position = ub_data.modelViewProjMatrix * vec4(inPosition, 1.0);

    // Calculate position in spotLight-space for shadows
    frag_out.fragPositionLightSpaceCS = ub_data.viewProjMatrixShadow * vec4(frag_out.fragPositionWS,1);
}
