#version 450

layout (location = 0) in vec3 in_position;
layout (location = 1) in vec3 in_normal;  // Input for normals

layout (binding = 0) uniform UniformBuffer {
    vec4 color;
    mat4 modelViewProjMatrix;
    mat4 modelMatrix;
    vec4 cameraPosition;
    ivec4 userInput;
    float ambientConstant;
    float diffuseConstant;
    float specularConstant;
    float shininessConstant;
} ub_data;

layout (binding = 1) uniform DirectionalLightBuffer {
    vec4 direction;
    vec4 color;
} dir_data;

layout (binding = 2) uniform PointLightBuffer {
    vec4 position;
    vec4 color;
    vec4 attenuation;
} point_data;

layout (location = 0) out vec3 fragNormal;     
layout (location = 1) out vec3 fragPosition;   
layout (location = 2) out vec4 fragColor;     

void main() {

    gl_Position = ub_data.modelViewProjMatrix * vec4(in_position, 1.0);
    // Normal and position in world space
    fragNormal = mat3(ub_data.modelMatrix) * in_normal;
    fragPosition = vec3(ub_data.modelMatrix * vec4(in_position, 1.0));

    // Original color
    fragColor = ub_data.color;

}
